/*global FileUploadOptions, FileTransfer, Camera, CameraPopoverOptions */
window.app.apps.communities = {
    initialised: false,
    communityPostImage: null,
    communityPostUrl: null,
    communityPostTitle: null,
    communityImage: null,
    destinationType: null,
    editingCommunityId: null,
    urlLiked: false,
    uploadingImage: false,
    uploadProgress: null,
    devApiRoute: 'http://api.kosmos.dev/community/',
    productionApiRoute: 'https://api.kosmos.co/community/',
    apiRoute: '',
    sendNewPostToCommunityData: null,
    xPullInstances: [],
    postToCommunityUrl: null,
    shareScrollButtonHidden: true,
    currentCommunityId: null,
    activeGif: null,
    currentShareUrl: null,

    init: function () {
        console.info('called communities.init()');

        /* For easier route switching */
        this.apiRoute = this.productionApiRoute;

        // Bind the event listeners
        document.addEventListener('backEvent', app.apps.communities._backEvent);
        document.addEventListener('beforeNavigateEvent', app.apps.communities._beforeNavigateEvent);
        document.addEventListener('pageLoadedEvent', app.apps.communities._pageLoadedEvent);
        document.addEventListener('beforeUrlCahngeEvent', app.apps.communities._beforeUrlChangeEvent);
        document.addEventListener('scroll', app.apps.communities.showShareScrollHandler);
        document.addEventListener('scroll', app.refresh.loadMoreHandler);
        // Attach message receivers
        app.apps.communities._addMessageReceivers();

        this._addMap();
        this.bindPullToRefresh();
        this.initialised = true;

        var $body = $('body');
        $body.delegate('.has-tap-state', 'touchstart', app.apps.communities.handleTouchState);
        $body.delegate('.has-tap-state', 'touchmove', app.apps.communities.handleTouchState);
        $body.delegate('.has-tap-state', 'touchend', app.apps.communities.handleTouchState);

        return this._getAppInfo();
    },

    handleTouchState: function () {

        if (event.type === 'touchstart') {
            this.classList.add('tapped');
        } else {
            this.classList.remove('tapped');
        }
    },

    _getAppInfo: function () {
        return {
            id: 'communities',
            name: 'Communities for people'
        };
    },

    _addMap: function () {
        app.refresh.addMap('page-communities-feed', {
            call: app.apps.communities.requestFeed,
            callback: app.apps.communities.requestFeedCallback,
            refreshPosition: 'innerHTML',
            loadMorePosition: 'beforeend',
            contentDivId: 'page-communities-feed-content',
            from: 0,
            limit: 10
        });
        app.refresh.addMap('page-communities', {
            call: app.apps.communities.loadCommunitiesList,
            callback: app.apps.communities.loadCommunitiesListCallback,
            refreshPosition: 'innerHTML',
            loadMorePosition: 'beforeend',
            contentDivId: 'communities-list-page-content',
            from: 0,
            limit: 10
        });
        app.refresh.addMap('page-communities-discover', {
            call: app.apps.communities.loadDiscoverCommunitiesList,
            callback: app.apps.communities.loadDiscoverCommunitiesListCallback,
            refreshPosition: 'innerHTML',
            loadMorePosition: 'beforeend',
            contentDivId: 'page-communities-discover-content',
            from: 0,
            limit: 10
        });
        app.refresh.addMap('page-browse-community', {
            call: app.apps.communities.renderCommunity,
            callback: app.apps.communities.renderCommunityCallback,
            refreshPosition: 'innerHTML',
            loadMorePosition: 'beforeend',
            contentDivId: 'page-browse-community-content',
            from: 0,
            limit: 10
        });
    },

    playGif: function (el, event) {
        event = event || window.event;

        if (!app.apps.communities.activeGif) {
            var elImg = el.getElementsByTagName('img');
                elImg = elImg ? elImg[0] : elImg;

            el.classList.add('active');
            elImg.src = elImg.getAttribute('data-gif');
            app.apps.communities.activeGif = elImg;

            document.addEventListener('touchstart', app.apps.communities.stopGif);
            event.stopPropagation();
        }
    },

    stopGif: function (el, event) {
        event = event || window.event;

        var elImg = app.apps.communities.activeGif;
            elImg.parentNode.classList.remove('active');
            elImg.src = elImg.getAttribute('data-snapshot');

        document.removeEventListener('touchstart', app.apps.communities.stopGif);
        elImg = null;
        app.apps.communities.activeGif = null;
        event.preventDefault();
        event.stopPropagation();
        return false;
    },

    showShareScrollHandler: util.debounce(function (event) {
        var currScroll, breakPointScroll = 58, shouldBeHidden, elShareHeaderButton;

        // we only do this logic in feed (aka home page)
        if (document.location.hash !== '#page-communities-feed') {
            return false;
        }

        currScroll = document.body.scrollTop;
        shouldBeHidden = currScroll < breakPointScroll;
        elShareHeaderButton = document.getElementById('share-header-button');

        if (shouldBeHidden && !app.apps.communities.shareScrollButtonHidden) {
            app.apps.communities.shareScrollButtonHidden = true;
            elShareHeaderButton.style.display = 'none';
        } else if (!shouldBeHidden && app.apps.communities.shareScrollButtonHidden) {
            app.apps.communities.shareScrollButtonHidden = false;
            elShareHeaderButton.style.display = '';
        }

    }, 200),

    bindPullToRefresh: function () {
        var idx, xPullEls, xPullEl;

        xPullEls = [
            'page-communities-feed-content-wrapper',
            'communities-list-page-content-wrapper',
            'page-communities-discover-content-wrapper'
        ];

        for (idx in xPullEls) {
            xPullEl = document.getElementById(xPullEls[idx]);

            // DOM not yet loaded with app html
            if (!xPullEl) {
                console.log('bindPullToRefresh did not find required DOM nodes - timeout will be set...');
                setTimeout(app.apps.communities.bindPullToRefresh, 400);
                return;
            }

            $(xPullEl).xpull({callback: app.refresh.reload});
            app.apps.communities.xPullInstances.push($(xPullEl).data('plugin_xpull'));
        }
    },

    loadDiscoverCommunitiesList: function (from, limit, insertPosition, cb) {
        cb = cb ? cb : function () {};
        from = from ? from : 0;
        limit = limit ? limit : 10;
        insertPosition = insertPosition ? insertPosition : 'innerHTML';

        if (insertPosition === 'innerHTML') {
            app.refresh.updateFrom(null, 'page-communities-discover', true);
        }

        var ajaxSettings = {
            url: app.apps.communities.apiRoute + 'discover',
            insertPosition: insertPosition,
            data: {
                from: from,
                limit: limit
            }
        };

        var onSuccess = cb;

        var onFail = function (response) {
            var data = util.getAjaxData(response);
            console.log('loadDiscoverCommunitiesList ajax error => ', data);
        };

        app.request.make(ajaxSettings, onSuccess, onFail);
    },

    loadDiscoverCommunitiesListCallback: function (response) {
        var data = util.getAjaxData(response);
        if (data.status === 'ok') {
            app.refresh.updateFrom(data.data.length, 'page-communities-discover');
            var showHeader = this.insertPosition === 'beforeend' ? false : true;
            ui.render('communities-discover', {discoverCommunities: data.data, showHeader: showHeader}, '#page-communities-discover-content', this.insertPosition);
        }
    },

    authorizedActions: {
        // action key: true -> show login / false -> show tooltip
        'you' : {
            accountAction: function () {
                app.tabs.open('you');
            },
            guestAction: function () {
                app.tabs.open('default');
            }
        },
        'notifications' : {
            accountAction: function () {
                app.tabs.open('notifications');
            },
            guestAction: function () {
                app.tabs.open('default');
            }
        },
        'votePost': {
            accountAction: function (communityId, postId, ownerId) {
                app.apps.communities.votePost(communityId, postId, ownerId);
                app.analytics.mx.track('community-post-like-loggedin');
            },
            guestAction: function () {
                app.apps.communities.toggleLoginTooltip(true);
                app.analytics.mx.track('community-post-like-guest');
            }
        },
        'loadPostToCommunity': {
            accountAction: function () {
                app.apps.communities.loadPostToCommunity();
                app.analytics.mx.track('click-post-compose-loggedin');
            },
            guestAction: function () {
                app.analytics.mx.track('click-post-compose-guest');
                app.apps.communities.toggleLoginTooltip(true);
            }
        },
        'createCommunity': {
            accountAction: function () {
                app.apps.communities.createCommunity();

            },
            guestAction: function () {
                app.apps.communities.toggleLoginTooltip(true);
            }
        },
        'join': {
            accountAction: function (communityId, communityTitle) {
                app.apps.communities.join(communityId, communityTitle);
            },
            guestAction: function () {
                app.apps.communities.toggleLoginTooltip(true);
            }
        },
        'postCommentUi': {
            accountAction: function () {
                // do nothin
            },
            guestAction: function (event) {
                if (event) {
                    event.stopPropagation();
                    event.preventDefault();
                }
                app.apps.communities.toggleLoginTooltip(true);
            }
        },
        'postComment': {
            accountAction: function () {
                app.apps.communities.postComment();
            },
            guestAction: function () {
                app.apps.communities.toggleLoginTooltip(true);
            }
        }
    },

    doAuthorizedAction: function (action) {
        var actionToUse, actionIsCallable, params;

        actionToUse = app.auth.getAccountId() === null ? 'guestAction' : 'accountAction';
        actionIsCallable = app.apps.communities.authorizedActions[action] && app.apps.communities.authorizedActions[action][actionToUse];

        if (actionIsCallable) {
            params = arguments;
            params = params ? Array.prototype.slice.call(arguments, 1) : params;

            if (params) {
                app.apps.communities.authorizedActions[action][actionToUse].apply(app.apps.communities, params);
            } else {
                app.apps.communities.authorizedActions[action][actionToUse]();
            }

        }
    },

    requestFeed: function (from, limit, insertPosition, cb) {
        cb = cb ? cb : function () {};
        from = from ? from : 0;
        limit = limit ? limit : 20;
        insertPosition = insertPosition ? insertPosition : null;
        var ajaxSettings = {
            url: app.apps.communities.apiRoute + 'feed',
            insertPosition: insertPosition,
            data: {
                from: from,
                limit: limit
            }
        };

        var onFail = function (response) {
            var data = util.getAjaxData(response);
            console.log('requestFeed ajax error => ', data);
        };

        app.request.make(ajaxSettings, cb, onFail);
    },

    requestFeedCallback: function (response) {
        var data = util.getAjaxData(response);
        if (data.status === 'ok') {
            //do not add share stuff when reloading content
            var showShare = this.insertPosition !== 'beforeend';
            //console.log('showShare', showShare);
            ui.render('communities-feed', {posts: data.data, showCommunityTitle: true, hideEnabled: false, showShare: showShare, profileAvatar: app.apps.profile.avatar}, '#page-communities-feed-content', this.insertPosition);

            var countExcludingPromoted = 0;
            data.data.filter(function (item) {
                if (!item.promoted) {
                    countExcludingPromoted++;
                }
            });
            app.refresh.updateFrom(countExcludingPromoted);
        }
    },

    showPost: function (identifier, postType) {
        app.navigate('page-community-post-item');

        var ajaxSettings = {
            url: app.apps.communities.apiRoute + 'get_post',
            identifier: identifier,
            postType: postType,
            data: {}
        };

        ajaxSettings.data[postType] = identifier;

        var handleView = function (response) {
            var data = util.getAjaxData(response);
            if (data.status === 'err') {
                data.data = {};
            }
            data.data.type = this.postType;
            data.data.identifier = this.identifier;
            ui.render('communities-post-item', data.data, '#page-community-post-item-content');
        };

        app.spinner.render('#page-community-post-item-content');

        app.request.make(ajaxSettings, handleView, handleView);
    },

    hidePost: function (postId, communityId, type) {
        var confirmMessage = type === 'activity' ? 'Do you really want to hide this from your activity?' : 'Do you really want to hide this post from this community?';

        function handleConfirm(buttonPressed) {
            // if buttonPressed code = 1 then canceled
            if (buttonPressed !== 1) {
                return false;
            }

            var hideUrl = type === 'activity' ? 'activity/hide' : 'post/hide';

            var ajaxSettings = {
                url: app.apps.communities.apiRoute + hideUrl,
                postId: postId,
                selector: '.community_post_item_' + postId,
                data: {
                    /* jshint -W106 */
                    post_id: postId,
                    community_id: communityId
                    /* jshint +W106 */
                }
            };

            var onSuccess = function () {
                $(this.selector).remove();
                plugins.toast.showShortBottom('Post successfully hidden.');

                var elPostsCount = document.getElementById('community-posts-count');

                if (elPostsCount) {
                    elPostsCount.innerHTML = elPostsCount.innerHTML - 1;
                }
            };

            var onFail = function (data) {
                plugins.toast.showShortBottom('Cannot hide post, please try again later');
                console.log('hidePost -> ajax error: ', data);
            };

            app.request.make(ajaxSettings, onSuccess, onFail);
        }

        navigator.notification.confirm(confirmMessage, handleConfirm, 'Hide post', ['Hide', 'Cancel']);
    },

    postToCommunitySubmit: function (id, communityName) {
        var ajaxSettings = {
            url: app.apps.communities.apiRoute + 'post',
            communityName: communityName,
            type: 'POST',
            data: {
                type: 'ur',
                /* jshint -W106 */
                community_id: id,
                image_url: app.apps.communities.communityPostImage,
                /* jshint +W106 */
                url: app.apps.communities.communityPostUrl,
                title: app.apps.communities.communityPostTitle
            }
        };

        var onSuccess = function (data) {
            data = util.getAjaxData(data);
            var msg;

            if (data.status === 'ok') {
                msg = 'Good work! You post successfully shared';
                plugins.toast.showShortBottom(msg);
            } else {
                msg = 'Cannot create a new post, please try again later';
                plugins.toast.showShortBottom(msg);
            }

            app.navigate('page-browser');
        };

        var onFail  = function (data) {
            console.log('postToCommunitySubmit -> error ', data);
        };

        app.request.make(ajaxSettings, onSuccess, onFail);
    },

    postToCommunityLinkPreview: function (url, cb) {
        var ajaxSettings = {
            url: app.apps.communities.apiRoute + 'link/preview',
            type: 'POST',
            data: {
                url: url
            }
        };

        var onSuccess = function (data) {
            data = util.getAjaxData(data);

            if (data.status === 'ok') {
                console.log('Preview data', data);
                cb(data.data);
            } else {
                console.log('Cannot get preview data');
            }
        };

        app.request.make(ajaxSettings, onSuccess);
    },

    createCommunity: function () {
        var focusTimmer;
        app.navigate('page-communities-new-community');
        var data = { mode: 'new', modeNew: true };
        ui.render('communities-edit', data, '#page-communities-new-community-content');

        function focusNewCommunityName() {
            var elToFocus = document.getElementById('title');

            if (elToFocus) {
                elToFocus.focus();

                if (cordova.plugins.Keyboard) {
                    cordova.plugins.Keyboard.show();
                }
            } else {
                if (window.location.hash === '#page-communities-new-community') {
                    focusTimmer = focusNewCommunityName();
                }
            }
        }

        focusTimmer = setTimeout(focusNewCommunityName, 400);
        app.analytics.mx.track('community-create-open');
    },

    createCommunitySubmit: function () {
        var formData = new FormData(document.getElementById('community-new-wrapper'));
        formData.append('account_id', app.auth.getAccountId());
        formData.append('account_hash', app.auth.getAccountHash());

        var ajaxSettings = {
            url: app.apps.communities.apiRoute + 'create',
            processData: false,
            contentType: false,
            type: 'POST',
            data: formData
        };

        app.spinner.render('#page-communities-new-community-content');

        var onSuccess = function (data) {
            var msg;
            data = util.getAjaxData(data);

            if (data.status === 'ok') {
                msg = 'Your new community successfully created';
                app.apps.communities._backEvent();
                app.refresh.reload();
                app.analytics.mx.track('community-create-success');
            } else {
                msg = 'Cannot create new community: "' + data.data + '"';
                app.apps.communities.createCommunity();
            }

            plugins.toast.showShortBottom(msg);
        };

        app.request.make(ajaxSettings, onSuccess);
    },

    loadPostToCommunity: function () {
        app.navigate('page-communities-new-post');

        function fillPostableCommunitiesUI(data) {
            var urlToShare, url, msg, linkEl, msgEl;

            ui.render('communities-select-community', {communities: data}, '#post-to-communities-list');

            urlToShare = app.storage.local.get('urlToShare');

            // Show login tooltip if user is not signed in
            if (app.auth.getAccountId() === null) {
                app.apps.communities.toggleLoginTooltip(true);
            }

            if (!urlToShare) {
                return false;
            }

            url = util.getLinks(urlToShare)[0];
            msg = urlToShare.replace(url, '').trim();

            setTimeout(function () {
                if (urlToShare) {
                    linkEl = document.getElementById('post-to-community-link');
                    msgEl = document.getElementById('post-to-community-message');
                    linkEl.value = url; msgEl.value = msg;
                    linkEl.focus();
                    setTimeout(app.apps.communities.validateSendToCommunity, 200, linkEl);
                }
            }, 400);
        }

        app.apps.communities.getPostableCommunities(fillPostableCommunitiesUI);
    },

    // @To-Do: implement functionality
    viewCommunitySettings: function (id, communityName, event) {
        app.navigate('page-communities-edit-community');
        app.spinner.render('#page-communities-edit-community-content');

        var ajaxSettings = {
            url: app.apps.communities.apiRoute + 'info',
            communityName: communityName,
            data: {
                /* jshint -W106 */
                community_id: id
                /* jshint +W106 */
            }
        };

        var onFail = function () {};
        var onSuccess = function (data) {
            var msg;
            data = util.getAjaxData(data);

            if (data.status === 'ok') {
                data.data.mode = 'edit';
                data.data.modeNew = false;
                /* jshint -W106 */
                data.data.canDelete = data.data.owned && !data.data.profile_community ? true : false;
                /* jshint +W106 */
                ui.render('communities-edit', data.data, '#page-communities-edit-community-content');
            } else {
                msg = 'Cannot open community settings: "' + data.data + '"';
                window.plugins.toast.showShortBottom(msg);
            }
        };

        app.request.make(ajaxSettings, onSuccess, onFail);
        if (event) {
            event.stopPropagation();
        }
    },

    updateCommunitySettings: function () {
        var postParams = {},
            i, len, form;

        // get post params from form
        form = document.getElementById('community-edit-wrapper');
        len = form.elements.length;

        for (i = 0; i < len; i++) {
            postParams[ form[i].name ] = form[i].value;
        }

        /* Generic callback (for both file transfer and ajax ) */
        var onSuccess = function (data) {
            var communityId = this.communityId;
            data = util.getAjaxData(data.response ? data.response : data);

            if (data.status === 'ok') {
                window.plugins.toast.showShortBottom('Communty settings successfully updated');
                app.history.reset();
                app.history.back();

                $(
                    [
                        '#page-browse-community-content .community-title',
                        '.community-item-box-' + communityId + ' .headline',
                        '#page-browse-community-header .inner span'
                    ].join(', ')
                ).text(this.updatedTitle);

                $(
                    [
                        '#page-browse-community-content .community-about',
                        '.community-item-box-' + communityId + ' .subheadline'
                    ].join(', ')
                ).text(this.updatedBio);
            } else {
                window.plugins.toast.showShortBottom(data.data);
                /* jshint -W106 */
                app.apps.communities.viewCommunitySettings(postParams.community_id, postParams.title);
                /* jshint +W106 */
            }
        };

        var onFail = function (data) {
            /* jshint -W106 */
            app.apps.communities.viewCommunitySettings(postParams.community_id, postParams.title);
            /* jshint +W106 */
            window.plugins.toast.showShortBottom('Server error occurred, please try again later');
            console.error('updateCommunitySettings ### data: ', data);
        };

        app.spinner.render('#page-communities-edit-community-content');

        var ajaxSettings = {
            url: app.apps.communities.apiRoute + 'update',
            /* jshint -W106 */
            communityId: postParams.community_id,
            /* jshint +W106 */
            updatedTitle: postParams.title,
            updatedBio: postParams.about,
            data: postParams
        };

        app.request.make(ajaxSettings, onSuccess, onFail);
    },

    uploadMedia: function (fileLocation) {
        var onFail = function (data) {
            console.log('Filetransfer fail with data', data, data.body);
            plugins.toast.showShortBottom('Saving failed');
        };

        var onSuccess = function (data) {
            console.log('Filetransfer onSuccess with data', data, data.response);
            var response = util.getAjaxData(data.response);
            if (response.status === 'ok') {
                plugins.toast.showShortBottom('Successfully saved');
                app.apps.communities.communityMediaUploadedHandler(response.data.path);
            } else {
                plugins.toast.showShortBottom('Cannot save: ' + response.data);
            }
        };

        var postParams = {
            destination: app.apps.communities.destinationType,
            /* jshint -W106 */
            community_id: app.apps.communities.editingCommunityId,
            account_id: app.auth.getAccountId(),
            account_hash: app.auth.getAccountHash()
            /* jshint +W106 */
        };

        // init file options
        var fileOptions = new FileUploadOptions(
            'image', // fileKey
            fileLocation, // fileName
            'image/jpeg', // mimeType
            postParams // params
        );

        var fileTransfer = new FileTransfer();
        fileTransfer.upload(
            fileLocation,
            encodeURI(app.apps.communities.apiRoute + 'update/media'),
            onSuccess,
            onFail,
            fileOptions
        );

        plugins.toast.showShortBottom('Uploading image...');
    },

    communityMediaUploadedHandler: function (uri) {
        var elVisualInEditView, elVisualOther;

        if (app.apps.communities.destinationType === 'banner') {
            elVisualInEditView = document.getElementById('community-banner');
        } else {
            elVisualInEditView = document.getElementById('community-avatar');
        }

        elVisualOther = $(app.apps.communities.destinationType === 'banner' ? '#banner-box' : '.community-item-box-' + app.apps.communities.editingCommunityId + ' .avatar');

        if (elVisualOther.length) {

            elVisualOther.css('background-image', 'url(' + uri + ')');
        }

        if (elVisualInEditView) {
            elVisualInEditView.style.backgroundImage = 'url(' + uri + ')';
        }
    },

    // make call via plugin to get photo from device
    getPhoneMedia: function (id, type) {
        this.destinationType = type;
        this.editingCommunityId = id;
        var options = {
            quality : 75,
            destinationType : Camera.DestinationType.FILE_URI,
            // DATA_URL: 0, Return image as base64-encoded string
            // FILE_URI: 1, Return image file URI
            // NATIVE_URI: 2 Return image native URI (e.g., assets-library:// on iOS or content://
            sourceType : Camera.PictureSourceType.PHOTOLIBRARY,
            allowEdit : true,
            encodingType: Camera.EncodingType.JPEG,
            popoverOptions: CameraPopoverOptions,
            saveToPhotoAlbum: false
        };

        function fail() {}

        navigator.camera.getPicture(app.apps.communities.uploadMedia, fail, options);
    },

    browseCommunity: function (communityId) {
        app.spinner.render('#page-browse-community');
        app.navigate('page-browse-community');
        app.refresh.updateFrom(null, 'page-browse-community', true);
        app.apps.communities.currentCommunityId = communityId;
        app.apps.communities.renderCommunity(null, null, 'innerHTML', app.apps.communities.renderCommunityCallback, communityId);
    },

    renderCommunity: function (from, limit, insertPosition, cb) {
        var communityId = app.apps.communities.currentCommunityId;
        cb = cb ? cb : function () {};
        from = from ? from : 0;
        limit = limit ? limit : 20;
        insertPosition = insertPosition ? insertPosition : null;
        var ajaxSettings = {
            url: app.apps.communities.apiRoute + 'info',
            insertPosition: insertPosition,
            data: {
                /* jshint -W106 */
                get_posts: true,
                community_id: communityId,
                /* jshint +W106 */
                from: from,
                limit: limit
            }
        };

        app.request.make(ajaxSettings, cb);
    },

    renderCommunityCallback: function (response) {
        var data = util.getAjaxData(response);
        if (data.status === 'ok') {
            app.refresh.updateFrom(data.data.posts.length);
            data.data.initLoad = this.insertPosition !== 'beforeend';
            data.data.hideEnabled = 'community';
            //console.log('Canhide', data.data.canHide);
            if (data.data.initLoad) {
                ui.render('communities-community', data.data, '#page-browse-community', this.insertPosition);
            } else {
                ui.render('communities-community', data.data, '#page-browse-community-content', this.insertPosition);
            }
        } else {
            // handle error from server here
            console.log('browserCommunity ajax error: ', data);
            plugins.toast.showShortBottom('Error occurred, please try again later');
        }
    },

    deleteCommunity: function (id, communityName) {
        var msg;
        communityName = communityName ? communityName : '';

        msg = 'Do you really want to delete "' + communityName + '" community?';

        function handleConfirm(buttonPressed) {
            // if buttonPressed code = 1 then canceled
            if (buttonPressed !== 1) {
                return false;
            }

            var ajaxSettings = {
                url: app.apps.communities.apiRoute + 'delete',
                communityName: communityName,
                communityId: id,
                /* jshint -W106 */
                data: { community_id: id }
                /* jshint +W106 */
            };

            var onFail = function () {};
            var onSuccess = function (data) {
                data = util.getAjaxData(data);

                if (data.status === 'ok') {
                    var msg = 'Community "' + this.communityName + '" successfuly removed.';
                    $('.community-item-box-' + this.communityId).remove();
                    app.navigate('page-communities');
                    window.plugins.toast.showShortBottom(msg);
                } else {
                    // handle errors here
                    console.log('deleteCommunity response with error handling here');
                }
            };

            app.request.make(ajaxSettings, onSuccess, onFail);
        }

        navigator.notification.confirm(msg, handleConfirm, 'Delete community', ['Delete', 'Cancel']);
    },

    loadCommunitiesList: function (from, limit, insertPosition, cb) {
        cb = cb ? cb : function () {};
        from = from ? from : 0;
        limit = limit ? limit : 20;
        insertPosition = insertPosition ? insertPosition : null;

        var ajaxSettings = {
            url: app.apps.communities.apiRoute + 'list',
            insertPosition: insertPosition,
            data: {
                from: from,
                limit: limit
            }
        };

        //app.spinner.render('#communities-list-page-content');
        app.request.make(ajaxSettings, cb);
    },

    loadCommunitiesListCallback: function (response) {
        var data = util.getAjaxData(response);
        if (data.status === 'ok') {
            var dataForTemplate = {
                userCommunities: data.data.own,
                joinedCommunities: data.data.joined
            };

            ui.render('communities-list', dataForTemplate, '#communities-list-page-content', this.insertPosition);
            app.apps.communities.cachePostableCommunities(data.data);
        } else {
            // handle error from server here
            console.log('error handling needs here');
        }
    },

    cachePostableCommunities: function (response, cb) {
        cb = cb ? cb : function () {};
        var postableCommunities = [];
        var allCommunities = response.joined.concat(response.own);
        /* jshint -W106 */
        for (var i = 0; i < allCommunities.length; i++) {
            if (allCommunities[i].can_post === true) {
                postableCommunities.push({
                    title: allCommunities[i].title,
                    id: allCommunities[i].community_id,
                    icon: allCommunities[i].icons.xsmall
                });
            }
        }
        /* jshint +W106 */
        cb(postableCommunities);
        app.storage.local.set('postableCommunities', JSON.stringify(postableCommunities));
    },

    getPostableCommunities: function (cb) {
        var postableCommunities = app.storage.local.get('postableCommunities');

        if (postableCommunities === null) {
            app.apps.communities.loadCommunitiesList(null, null, null, cb);
            // return since we cant execute cb now
            return false;
        }

        postableCommunities = JSON.parse(postableCommunities);
        cb(postableCommunities);
    },

    inputEventHandlers: {
        footer: document.getElementById('page-community-post-item-footer'),
        elInput: document.getElementById('community-post-message'),

        focus: function () {
            this.footer.classList.add('focused');
        },
        blur: function () {
            if (!this.elInput.value.trim()) {
                this.footer.classList.remove('active');
                this.footer.classList.remove('focused');
            }
        },
        checkState: function (el) {
            var footerActive = this.footer.classList.contains('active');

            if (el.value.trim() && !footerActive) {
                this.footer.classList.add('active');
            } else if (!el.value.trim() && footerActive) {
                this.footer.classList.remove('active');
            }
        },
        keyup: function (el, event) {
            if (event.charCode === 13) {
                app.apps.communities.postComment();
            }
        },
        input: function (el, event) {
            this.checkState(el, event);
        },
        clear: function () {
            this.elInput.value = '';
            this.elInput.blur();
        }
    },

    handleInputEvents: function (el, event) {
        if (app.apps.communities.inputEventHandlers[event.type]) {
            app.apps.communities.inputEventHandlers[event.type](el, event);
        } else {
            console.error('handleInputEvents event type w/o event handler occurred');
        }
    },

    postCommentSuccess: function (data) {
        var elCommentsCount;

        if (data.status === 'ok') {
            data.data.comment = ui.escapeHTML(data.data.comment);
            /* jshint -W106 */
            data.data.id = data.data.comment_id;
            data.data.author_name = app.apps.profile.getNickname();
            /* jshint +W106 */

            var placehodlerEl = $('.discuss_welcome_placeholder');
            /* jshint -W106 */
            data.data.likes_count = data.data.likes_count || '0';
            /* jshint +W106 */
            if (placehodlerEl.length) {
                ui.render('communities-community-single-comment', data.data, placehodlerEl, 'outerHTML');
            } else {
                ui.render('communities-community-single-comment', data.data, '#discussion-comments-wrapper', 'beforeend');
            }

            // update comments counts
            if (this.postId) {
                /* jshint -W106 */
                elCommentsCount = $('.post-' + this.postId +  '-comments-count-text');
                /* jshint +W106 */
                app.apps.communities.updateSocialScore('comment', elCommentsCount);
            }


        } else {
            plugins.toast.showShortBottom('Error occurred, please try again later');
            console.log('Comment was not posted', data);
        }
    },

    postComment: function (comment) {
        var type, value, emptyComment, communityId;
        /* type is either 'post_id' or 'url' */

        type = document.getElementById('post-item-type').value;
        value = document.getElementById('post-item-indetifier').value;
        communityId = document.getElementById('post-item-community-id').value;

        comment = comment || document.getElementById('community-post-message').value;
        emptyComment = !(comment = comment.trim());
        if (emptyComment) {
            plugins.toast.showShortBottom('Please write a comment');
            console.error('Trying to post comment on community item / url with empty comment.');
            return false;
        }

        var ajaxSettings = {
            url: app.apps.communities.apiRoute + 'comment',
            data: {
                comment: comment
            }
        };

        ajaxSettings.data[type] = value;

        /* jshint -W106 */
        if (type === 'url') {
            ajaxSettings.data.image_url = app.apps.communities.communityPostImage;
            ajaxSettings.data.title = app.apps.communities.communityPostTitle;
            app.analytics.mx.track('communities-posted-comment-on-url', {url: value});
        } else if (type === 'post_id') {
            ajaxSettings.data.community_id = communityId;
            ajaxSettings.postId = value;
            app.analytics.mx.track('communities-posted-comment-on-post', {post_id: value});
        }
        /* jshint +W106 */

        app.apps.communities.inputEventHandlers.clear();
        app.request.make(ajaxSettings, app.apps.communities.postCommentSuccess);
    },

    updateSocialScore: function (action, el) {
        var score, actionsValues, newScore, zeroSymbol = '';

        actionsValues = { like: 1, comment: 1, unlike: -1 };

        // grab current score and try to parse it as int
        score = parseInt(el[0].innerText, 10);
        // if it fails to covert to int - assume its 0
        score = isNaN(score) ? 0 : score;

        // add on like - sub on unlike
        newScore = score;
        newScore = newScore + actionsValues[action];
        // instead of showing 0 we use '-' char
        newScore = newScore <= 0 ? zeroSymbol : newScore;

        el.text(newScore);
    },

    voteComment: function (commentId, elBox) {
        var ajaxSettings, onSuccess, action;

        elBox = $(elBox);
        action = elBox.hasClass('liked') ? 'unlike' : 'like';

        ajaxSettings = {
            url: app.apps.communities.apiRoute + action,
            type: 'POST',
            commentId: commentId,
            postAction: action,
            elBox: elBox,
            /* jshint -W106 */
            data: { type: 'cc', comment_id: commentId }
            /* jshint +W106 */
        };

        onSuccess = function () {};

        elBox.toggleClass('liked');
        app.apps.communities.updateSocialScore(action, elBox.find('.likes_counter'));

        app.request.make(ajaxSettings, onSuccess);
    },

    votePost: function (communityId, postId, ownerId) {
        var ajaxSettings, onSuccess, action, profileCommunity, hideOnUnlike, elBox;

        elBox = $('.community_post_item_' + postId);
        action = elBox.first().hasClass('liked') ? 'unlike' : 'like';
        profileCommunity = localStorage.getItem('profileCommunityId') === communityId;
        hideOnUnlike = (ownerId === app.auth.getAccountId() && profileCommunity);

        /* jshint -W106 */
        ajaxSettings = {
            url: app.apps.communities.apiRoute + action,
            type: 'POST',
            postId: postId,
            elBox: elBox,
            profileCommunity: profileCommunity,
            hideOnUnlike: hideOnUnlike,
            voteAction: action,
            data: {
                type: 'cp',
                post_id: postId,
                community_id: communityId
            }
        };
        /* jshint +W106 */

        onSuccess = function (data) {};

        elBox.toggleClass('liked');

        if (hideOnUnlike) {
            elBox.remove();
        } else {
            app.apps.communities.updateSocialScore(action, elBox.find('.likes_count'));
        }

        app.request.make(ajaxSettings, onSuccess);
    },

    join: function (communityId, communityTitle) {
        function handleSuccess() {
            plugins.toast.showShortBottom('You have successfully joined "' + communityTitle);
            /* jshint -W106 */
            app.apps.communities.loadCommunitiesList(null, null, null, app.apps.communities.loadCommunitiesListCallback);
            /* jshint +W106 */
            app.analytics.mx.track('app-community-join');
        }

        var ajaxSettings = {
            url: app.apps.communities.apiRoute + 'join',
            /* jshint -W106 */
            data: { community_id: communityId }
            /* jshint +W106 */
        };

        var onFail = function () {
            plugins.toast.showShortBottom('Error occurred, please try again later');
        };

        var onSuccess = function (data) {
            data = util.getAjaxData(data);

            if (data.status === 'ok') {
                /* jshint -W106 */
                handleSuccess(data);
                /* jshint +W106 */
            } else {
                onFail();
            }
        };

        var elMetaItemBox = document.getElementById('community-meta-item-box');

        if (elMetaItemBox) {
            elMetaItemBox.classList.add('joined');
        }

        var elPromoButton = event && event.target && event.target.classList.contains('promo-button') ? event.target : null;

        if (elPromoButton) {
            elPromoButton.style.display = 'none';
        }

        app.request.make(ajaxSettings, onSuccess, onFail);
    },

    leave: function (communityId) {
        var ajaxSettings = {
            url: app.apps.communities.apiRoute + 'leave',
            /* jshint -W106 */
            data: { community_id: communityId }
            /* jshint +W106 */
        };

        var onFail = function () {
            plugins.toast.showShortBottom('Error occurred, please try again later');
        };

        var onSuccess = function (data) {
            data = util.getAjaxData(data);

            if (data.status === 'ok') {
                plugins.toast.showShortBottom('Community left successfully!');
                /* jshint -W106 */
                app.apps.communities.loadCommunitiesList(null, null, null, app.apps.communities.loadCommunitiesListCallback);
                /* jshint +W106 */
                app.analytics.mx.track('app-community-leave');
            } else {
                onFail();
            }
        };

        document.getElementById('community-meta-item-box').classList.remove('joined');
        app.history.back();
        app.request.make(ajaxSettings, onSuccess, onFail);
    },

    leaveDialog: function (communityId, communityTitle) {
        var action = function (e) {
            //do nothing if user pressed no or cancelled dialog
            if (e !== 1) {
                return false;
            }
            app.apps.communities.leave(communityId);
        };
        navigator.notification.confirm(
            'Do you really want to leave "' + communityTitle + '" community?',
            action,
            'Confirm action',
            ['Leave', 'No']
        );
    },

    discoverSearchInputHandler: function (event) {
        var handler = {};

        handler.blur = function () {
            app.apps.communities.toggleDiscoverSearch(false);
        };

        handler.keyup = function (event) {
            var text, textValid, enterPressed;

            text = event.target.value.trim();
            textValid = 3 < text.length;
            enterPressed = event.keyIdentifier && event.keyIdentifier === 'Enter';

            if (textValid && enterPressed) {
                app.apps.communities.discoverSearchRequest(text);
            }
        };

        if (handler[event.type]) {
            handler[event.type](event);
        }
    },

    toggleDiscoverSearch: function (toggleValue) {
        var elBox, elInput, currentState;

        elBox = document.getElementById('discover-search-box');
        elInput = document.getElementById('discover-search-input');
        currentState = elBox.style.display !== 'none';

        if (toggleValue === true && !currentState) {
            elBox.style.display = '';
            elInput.focus();
        } else if (toggleValue === false  && currentState) {
            elBox.style.display = 'none';
        } else {
            if (currentState) {
                elBox.style.display = 'none';
            } else {
                elBox.style.display = '';
                elInput.focus();
            }
        }
    },

    discoverSearchRequest: function (text) {
        var ajaxSettings = {
            url: app.apps.communities.apiRoute + 'search',
            searchKey: text,
            data: { key: text }
        };

        var onFail = function () {
            plugins.toast.showShortBottom('Network error occurred');
        };

        var onSuccess = function (data) {
            data = util.getAjaxData(data);

            if (data.status === 'ok') {
                if (data.data.length === 0) {
                    plugins.toast.showShortBottom('No communities found');
                } else {
                    var dataForRender = {
                        searchKey: this.searchKey,
                        discoverCommunities: data.data
                    };

                    ui.render('communities-discover', dataForRender, '#page-communities-discover-content');
                }
            } else {
                plugins.toast.showShortBottom(data.data);
            }
        };

        app.request.make(ajaxSettings, onSuccess, onFail);
    },

    getUserActivity: function (activityOf, from, limit, insertPosition, subpage) {
        from = from ? from : 0;
        limit = limit ? limit : 20;
        insertPosition = insertPosition ? insertPosition : 'innerHTML';
        subpage = subpage ? subpage : 'activity';

        var ajaxSettings = {
            profileId: activityOf,
            insertPosition: insertPosition,
            subpage: subpage,
            data: {
                from: from,
                limit: limit
            }
        };

        if (subpage === 'activity') {
            /* jshint -W106 */
            ajaxSettings.url = app.apps.communities.apiRoute + 'activity';
            ajaxSettings.data.activity_of = activityOf;
            /* jshint +W106 */
        } else {
            ajaxSettings.url = app.apps.communities.apiRoute + 'likes';
        }

        app.request.make(ajaxSettings, app.apps.communities.renderUserActivity);
    },

    // used to both render ownProfile and publicProfiles
    renderUserActivity: function (data) {
        var elRoot, elFeedBox, elPostsCounter, elMembersCounter, isOwnProfile, activity, pageHash;

        isOwnProfile = app.auth.isOwnProfile(this.profileId);
        activity = this.subpage;

        if (isOwnProfile) {
            pageHash = activity === 'activity' ? 'page-app-profile' : 'page-app-profile-likes-activity';
            elRoot = document.getElementById(activity === 'activity' ? 'profile-info' : 'profile-info-likes-activity');
        } else {
            pageHash = 'page-app-profile-public';
            elRoot = document.getElementById('profile-public-info');
        }

        if (data.status === 'err') {
            console.error('Error while getting user activity', data);
            return false;
        }

        app.refresh.updateFrom(data.data.length, pageHash);

        elFeedBox = elRoot.getElementsByClassName('profile-feed-box')[0];
        elPostsCounter = elRoot.getElementsByClassName('postsCounter')[0];
        elMembersCounter = elRoot.getElementsByClassName('membersCounter')[0];

        ui.render('communities-feed', { posts: data.data, subpage: this.subpage, showCommunityTitle: true, hideEnabled: 'activity' }, elFeedBox, this.insertPosition);

        /* jshint -W106 */
        if (elPostsCounter && data.data.posts_count) {
            elPostsCounter.innerText = data.data.posts_count;
            /* jshint +W106 */
        }
        if (elMembersCounter && data.data.members) {
            elMembersCounter.innerText = data.data.members;
        }
    },

    showPostLinkPreview: function (title, imageUrl) {
        ui.render('communities-preview', {title: title, image: imageUrl}, '#preview-box');
    },

    resetPostLinkPreview: function () {
        var previewBox = document.getElementsByClassName('preview-box');

        previewBox = previewBox && previewBox.length > 0 ? previewBox[0] : null;
        if (previewBox) {
            //reset previewBot
            previewBox.innerHTML = '';
        }
    },

    validateSendToCommunity: function (el) {
        var link, isUrl, communityId, elPage, postValid, hasClass;
        var message = document.getElementById('post-to-community-message').value.trim();
        el = el || document.getElementById('post-to-community-link');
        link = el.value.trim();
        isUrl = util.isUrl(link);
        communityId = document.getElementById('post-to-community_id').value;
        elPage = document.getElementById('page-communities-new-post');
        postValid = link && isUrl && communityId;
        hasClass = elPage.classList.contains('valid');
        console.log(isUrl, link);
        if (postValid) {
            if (link !== app.apps.communities.postToCommunityUrl) {
                //try to load preview
                app.apps.communities.postToCommunityLinkPreview(link, function (preview) {
                    /* jshint -W106 */
                    app.apps.communities.showPostLinkPreview(preview.title, preview.image_url);
                    console.log('Preview data', preview.title, preview.image_url);
                    /* jshint +W106 */
                });
                //save typed url
                app.apps.communities.postToCommunityUrl = link;
            } else {
                app.apps.communities.postToCommunityUrl = null;
            }
        } else {
            app.apps.communities.resetPostLinkPreview();
            app.apps.communities.postToCommunityUrl = null;
        }

        /* jshint -W106 */
        app.apps.communities.sendNewPostToCommunityData = postValid ? {url: link, message: message, community_id: communityId } : null;
        /* jshint -W106 */

        if (postValid && !hasClass) {
            elPage.classList.add('valid');
        } else if (!postValid && hasClass) {
            elPage.classList.remove('valid');
        }
    },

    sendNewPostToCommunity: function () {
        var ajaxSettings = {
            url: this.apiRoute + 'post',
            dataSent: app.apps.communities.sendNewPostToCommunityData,
            data: app.apps.communities.sendNewPostToCommunityData
        };

        var onSuccess = function (data) {};

        var onError = function () {
            plugins.toast.showShortBottom('Network error occurred, please try again');
        };

        plugins.toast.showShortBottom('Your post successfully shared');
        app.history.back();
        app.apps.communities.clearPostToCommunity();

        app.request.make(ajaxSettings, onSuccess, onError);
    },

    clearPostToCommunity: function () {
        document.getElementById('post-to-community-message').value = '';
        document.getElementById('post-to-community-link').value = '';
        app.apps.communities.validateSendToCommunity();
        app.apps.communities.selectCommunityForPost(false);
    },

    selectCommunityForPost: function (communityId, icon, title) {
        icon = icon === 'false' ? false : icon;
        var page = document.getElementById('page-communities-new-post');
        var elIcon = document.getElementById('selected-community-icon');
        var elLabel = document.getElementById('selected-community-label');
        var elCommunityId = document.getElementById('post-to-community_id');
        // turn off 'community-selected' mode if no param given or first param falsy
        if (!communityId && communityId !== 0) {
            page.classList.remove('community-selected');
            elIcon.style.backgroundImage = 'none';
            elLabel.innerText = '';
            elCommunityId.value = '';
        } else {
            elIcon.style.backgroundImage = icon ? ('url(' + icon + ')') : 'none';
            elLabel.innerText = title;
            page.classList.add('community-selected');
            elCommunityId.value = '' + communityId;
        }

        app.apps.communities.validateSendToCommunity();
    },

    openBrowser: function (url) {
        app.navigate('page-browser');
        app.apps.communities.updateBrowserTitle('Loading...');
        app.apps.communities.currentShareUrl = url;
        app.browser.webview.show();
        app.browser.webview.nav().go(url, false, true);
    },

    updateBrowserTitle: function (title) {
        document.getElementById('browser-title').innerText = title;
    },

    resetBrowser: function () {
        app.browser.webview.nav().go('about:blank');
        app.browser.webview.hide();
    },

    toggleLoginTooltip: function (state, event) {
        var el;

        el = document.getElementById('login-tooltip-box');
        el.style.display = state === true ? '' : 'none';

        if (event && event.stopPropagation) {
            event.stopPropagation();
        }
    },

    //////////////////////////////////////////////////
    //              MESSAGE RECEIVERS               //
    //////////////////////////////////////////////////

    _addMessageReceivers: function () {
    },

    //////////////////////////////////////////////////
    //              pageLoadedEvent LISTENERS       //
    //////////////////////////////////////////////////
    _pageLoadedEvent: function (e) {
        app.apps.communities.updateBrowserTitle(e.data.title);
    },

    _beforeUrlChangeEvent: function (e) {
    },

    //////////////////////////////////////////////////
    //              NAVIGATION LISTENERS            //
    //////////////////////////////////////////////////

    _beforeNavigateEvent: function (e) {

        // If we are going to see offline page we need to enable webview
        if (e.data.navigateTo === '') {
            app.browser.webview.show();
        }
        // If we are going to see the list of offline pages we need to make sure that the browser is hidden
        if (e.data.navigateTo === '') {
            app.apps.communities.list();
        }

        // Clean shareUrl if back pressed
        if (e.data.navigateFrom === '#page-communities-new-post') {
            app.storage.local.del('urlToShare');
        }
    },

    //////////////////////////////////////////////////
    //              BACK BUTTON LISTENERS           //
    //////////////////////////////////////////////////

    _backEvent: function (e) {

        if (location.hash.indexOf('#page-browser') !== -1) {
            app.apps.communities.resetBrowser();
        }

        if (location.hash.indexOf('#page-communities') !== -1 || location.hash.indexOf('#page-community') !== -1 ||
            location.hash.indexOf('#page-post-to-community') !== -1 || location.hash.indexOf('#page-browse-community') !== -1 ||
            location.hash.indexOf('#page-browser') !== -1) {
            if (location.hash.indexOf('') !== -1) {
                if (e) {
                    e.preventDefault();
                }
                app.history.back();
            }
        }
    }
};
