window.app.apps.notifications = {
    limit: 20,
    refreshInterval: null,

    init: function () {
        console.info('called notifications.init()');

        // Bind the event listeners
        document.addEventListener('backEvent', app.apps.notifications._backEvent);
        document.addEventListener('beforeNavigateEvent', app.apps.notifications._beforeNavigateEvent);
        // Attach message receivers
        app.apps.notifications._addMessageReceivers();

        app.apps.notifications._populateSchema();

        app.apps.notifications._addMap();
        app.apps.notifications._initNotificationsUpdater();
        return this._getAppInfo();
    },

    _getAppInfo: function () {
        return {
            id: 'notifications',
            name: 'display notifications for users'
        };
    },

    _populateSchema: function () {
        console.log('Creating community_notifications table');
        app.storage.sql.exec([
            //'DROP TABLE IF EXISTS notifications',
            'CREATE TABLE IF NOT EXISTS community_notifications (id INTEGER PRIMARY KEY, icon TEXT, message TEXT, action TEXT, created_at INTEGER, read INTEGER)'
        ]);
    },

    _addMap: function () {
        app.refresh.addMap('page-app-notifications-list', {
            call: app.apps.notifications.readNotifications,
            callback: app.apps.communities.readNotificationsCallback,
            refreshPosition: 'innerHTML',
            loadMorePosition: 'beforeend',
            contentDivId: 'notifications-messages',
            from: 0,
            limit: 20
        });
    },

    _initNotificationsUpdater: function () {
        console.log('Starting notifications updater');
        app.apps.notifications.refreshInterval = setInterval(app.apps.notifications.getNotificationsFromServer, 60 * 1000); // one minute interval
    },

    getNotificationsFromServer: function () {
        var onSuccess = function (res) {
            if (res.status === 'ok' && res.data.length) {
                app.apps.notifications.writeNotificationsToDatabase(res.data);
            }
        };

        var onFail = function (res) {
            console.log('Notifications get fail callback', res);
        };

        var ajaxSettings = {
            url: 'https://api.kosmos.co/community/notifications',
            type: 'POST',
            data: {
                /* jshint -W106 */
                last_time: app.apps.notifications.getNotificationLastTime(),
                limit: 20
                /* jshint +W106 */
            }
        };

        if (app.auth.getAccountId() !== null) {
            app.request.make(ajaxSettings, onSuccess, onFail, 20000, 3, true);
        }
    },

    writeNotificationsToDatabase: function (notifications) {
        var lastTime = 0, currNotification, unread = [];

        for (var key in notifications) {
            // Cache curr notification
            currNotification = notifications[key];

            console.log('Writing notification to DB: ', currNotification);
            /* jshint -W106 */
            app.storage.sql.exec([[
                'INSERT OR REPLACE INTO community_notifications (id, icon, message, action, created_at, read) VALUES (?,?,?,?,?,?)',
                [
                    currNotification.notification_id,
                    currNotification.icon,
                    currNotification.text,
                    JSON.stringify(currNotification.action),
                    (currNotification.created_at),
                    currNotification.is_read
                ]
            ]]);

            if (currNotification.created_at > lastTime) {
                lastTime = currNotification.created_at;
            }

            // Collect all unread notifications
            if (currNotification.is_read === '0') {
                unread.push(currNotification);
            }
            /* jshint +W106 */
        }

        app.apps.notifications.updateNotificationRead(lastTime);
        app.refresh.reload('page-app-notifications-list');

        // Display last unread notification
        if (unread.length) {
            app.apps.notifications.showBubble();
        }
    },

    updateNotificationRead: function (time) {
        if (typeof time !== 'undefined' && time > 0) {
            app.storage.local.set('notifications_last_time', time);
        }
    },

    getNotificationLastTime: function () {
        var lastTime = app.storage.local.get('notifications_last_time');
        if (lastTime === null) {
            lastTime = 1;
        }

        return lastTime;
    },

    handleNotificationAction: function (el) {
        if (el.getAttribute('data-action')) {
            var dataAction = el.getAttribute('data-action');
            var dataValue = el.getAttribute('data-value');
            if (dataAction && dataValue) {
                if (dataAction === 'openBrowser') {
                    app.apps.communities.openBrowser(dataValue);
                    return true;
                }
            }
        }
        return false;
    },

    showBubble: function () {
        var footer = document.getElementById('tab-notifications-icon');
        if (!footer.classList.contains('has-new-notifications')) {
            footer.classList.add('has-new-notifications');
        }
    },

    hideBubble: function () {
        var footer = document.getElementById('tab-notifications-icon');
        if (footer.classList.contains('has-new-notifications')) {
            footer.classList.remove('has-new-notifications');
        }
    },

    readNotifications: function (from, limit, insertPosition, cb) {
        cb = cb ? cb : app.apps.notifications.readNotificationsCallback;
        //from = from ? from : 0;
        limit = limit ? limit : 20;
        //insertPosition = insertPosition ? insertPosition : null;

        //app.spinner.render('#notifications-messages');
        app.storage.sql.read('SELECT * FROM community_notifications ORDER BY created_at DESC LIMIT ?', [limit], cb);
    },

    readNotificationsCallback: function (res) {
        var data;
        if (res.length) {
            for (var i = 0; i < res.length; i++) {
                var action = JSON.parse(res[i].action);
                res[i].dataAction = action.action ? action.action : '';
                res[i].dataValue = action.value ? action.value : '';
            }

            data = { notifications: res };
            //@todo insert position
            ui.render('notifications-list', data, '#notifications-messages');

            // used to filter out read notifications with biggest id
            var pickUnread = function (notif) {
                return notif.read === 0;
            };
            /* jshint -W106 */
            var pickBigger = function (a, b) {
                if (a.created_at < b.created_at) {
                    return -1;
                }
                if (a.created_at > b.created_at) {
                    return 1;
                }
                return 0;
            };

            // get last (with biggest id) unread notification
            var lastUnread = res.filter(pickUnread).sort(pickBigger).pop();
            //console.log('lastUnread', lastUnread);
            if (lastUnread) {
                app.apps.notifications.markAsReadOnServer(lastUnread.created_at, true);
            }
            /* jshint +W106 */
        } else {
            data = {
                /* jshint -W106 */
                title: 'No notifications yet',
                subtitle: 'Go discover some communities better instead!',
                font_icon: 'notifications'
                /* jshint +W106 */
            };

            ui.render('page-no-content', data, '#notifications-messages');
        }
    },

    markNotificationsAsRead: function () {
        app.storage.sql.exec([[
            'UPDATE community_notifications SET read=1'
        ]]);
    },

    markAsReadOnServer: function (item, multiple) {
        /* jshint -W106 */
        var data = {};

        if (multiple) {
            data.last_time = item;
        } /*else {
            data.single = item;
        }*/

        /* jshint +W106 */
        var ajaxSettings = {
            url: app.apps.communities.apiRoute + 'notifications/mark',
            data: data
        };
        app.request.make(ajaxSettings, function () {}, null, 20000, 3, true);
    },

    clearNotifications: function () {
        app.storage.sql.exec([[
            'DELETE FROM community_notifications',
            []
        ]], function () {
            app.refresh.reload('page-app-notifications-list');
            var ajaxSettings = {
                url: app.apps.communities.apiRoute + 'notifications/delete'
            };
            app.request.make(ajaxSettings, function () {}, null, 20000, 3, true);
        });
    },

    //////////////////////////////////////////////////
    //              MESSAGE RECEIVERS               //
    //////////////////////////////////////////////////

    _addMessageReceivers: function () {
    },

    //////////////////////////////////////////////////
    //              NAVIGATION LISTENERS            //
    //////////////////////////////////////////////////

    _beforeNavigateEvent: function (e) {
    },


    //////////////////////////////////////////////////
    //              BACK BUTTON LISTENERS           //
    //////////////////////////////////////////////////

    _backEvent: function (e) {
        if (location.hash.indexOf('#page-app-notifications') !== -1) {
            if (e) {
                e.preventDefault();
            }
            app.history.back();
        }
    }
};
