/*global FileUploadOptions, FileTransfer, Camera, CameraPopoverOptions */
window.app.apps.profile = {
    initialised: false,
    apiRoute: 'https://api.kosmos.co/profile/',
    //apiRoute: 'http://api.kosmos.dev/profile/',
    nickname: null,
    avatar: null,
    newProfilePhoto: null,
    currentProfileId: null,

    init: function () {
        console.info('called profile.init()');

        if (!window.util.getAjaxData) {
            window.util.getAjaxData = function (data) {
                if (typeof data === 'string' || data instanceof String) {
                    data = JSON.parse(data);
                }
                if (data.status) {
                    data.status = data.status.toLowerCase();
                }

                return data;
            };
        }

        // Bind the event listeners
        document.addEventListener('backEvent', app.apps.profile._backEvent);
        document.addEventListener('beforeNavigateEvent', app.apps.profile._beforeNavigateEvent);
        // Attach message receivers
        app.apps.profile._addMessageReceivers();

        app.apps.profile.requestInitProfileData();

        this.bindPullToRefresh();
        this._addMap();

        this.initialised = true;

        return this._getAppInfo();
    },

    _getAppInfo: function () {
        return {
            id: 'profile',
            name: 'profile of user using app'
        };
    },

    _addMap: function () {
        app.refresh.addMap('page-app-profile', {
            call: app.apps.profile.getOwnProfile,
            callback: app.apps.profile.renderOwnProfile,
            refreshPosition: 'innerHTML',
            loadMorePosition: 'beforeend',
            spinnerPositionSelector: '#profile-info .profile-feed-box',
            contentDivId: 'profile-info',
            subpage: 'activity',
            from: 0,
            limit: 10
        });
        app.refresh.addMap('page-app-profile-public', {
            call: app.apps.profile.loadPublicProfile,
            callback: app.apps.profile.loadPublicProfileCallback,
            refreshPosition: 'innerHTML',
            loadMorePosition: 'beforeend',
            spinnerPositionSelector: '#profile-public-info .profile-feed-box',
            contentDivId: 'profile-public-info',
            from: 0,
            limit: 10
        });
        app.refresh.addMap('page-app-profile-likes-activity', {
            call: app.apps.profile.getOwnProfile,
            callback: app.apps.profile.renderOwnProfile,
            refreshPosition: 'innerHTML',
            loadMorePosition: 'beforeend',
            spinnerPositionSelector: '#profile-info-likes-activity .profile-feed-box',
            contentDivId: 'profile-info-likes-activity',
            subpage: 'likes',
            from: 0,
            limit: 10
        });
    },

    bindPullToRefresh: function () {
        var idx, xPullEls, xPullEl;

        xPullEls = [
            'page-app-profile-content-wrapper',
            'page-app-profile-likes-activity-content-wrapper'
        ];

        for (idx in xPullEls) {
            xPullEl = document.getElementById(xPullEls[idx]);

            // DOM not yet loaded with app html
            if (!xPullEl) {
                console.log('bindPullToRefresh did not find required DOM nodes - timeout will be set...');
                setTimeout(app.apps.communities.bindPullToRefresh, 400);
                return;
            }

            $(xPullEl).xpull({ callback: app.refresh.reload });

            app.apps.communities.xPullInstances.push($(xPullEl).data('plugin_xpull'));
        }
    },

    //////////////////////////////////////////////////
    //              MESSAGE RECEIVERS               //
    //////////////////////////////////////////////////
    _addMessageReceivers: function () {
    },

    catchSubmitKey: function (el, event) {
        if (event.keyCode === 13) {
            document.getElementById('profile_submit').click();
        }
    },

    // make call via plugin to get photo from device
    requestProfileImage: function () {
        var options = {
            quality : 75,
            destinationType : Camera.DestinationType.FILE_URI,
            // DATA_URL: 0, Return image as base64-encoded string
            // FILE_URI: 1, Return image file URI
            // NATIVE_URI: 2 Return image native URI (e.g., assets-library:// on iOS or content://
            sourceType : Camera.PictureSourceType.PHOTOLIBRARY,
            allowEdit : true,
            encodingType: Camera.EncodingType.JPEG,
            popoverOptions: CameraPopoverOptions,
            saveToPhotoAlbum: false
        };

        function success(uri) {

            var confirmed = function (e) {
                //do nothing if user pressed no or cancelled dialog
                if (e !== 2) {
                    return false;
                }

                app.apps.profile.newProfilePhoto = uri;
                app.apps.profile.updateProfilePicture();
            };

            navigator.notification.confirm(
                'Do you really want to upload selected photo?',
                confirmed,
                'Confirm selected photo',
                ['Cancel', 'Upload']
            );
        }

        function fail() {}

        navigator.camera.getPicture(success, fail, options);
    },

    updateProfilePicture: function () {
        var fileOptions, fileTransfer;

        document.getElementById('owner_avatar').style.backgroundImage = '';
        app.spinner.render('#owner_avatar', 'innerHTML', false);

        var success = function (data) {
            data = util.getAjaxData(data.response ? data.response : data);

            if (data.status === 'ok') {
                document.getElementById('owner_avatar').setAttribute('style', 'background-image: url(' + app.apps.profile.newProfilePhoto + ');');
                document.getElementById('owner_avatar').innerHTML = '';
                app.apps.profile.newProfilePhoto = null;
                window.plugins.toast.showShortBottom('Community settings successfully changed');
            } else {
                window.plugins.toast.showShortBottom(data.data);
                console.log('updateProfilePicture returned error -> ', data.data);
            }
        };

        var error = function (data) {
            window.plugins.toast.showShortBottom('Server error occurred');
            console.error('updateProfilePicture ### data: ', data);
        };

        fileOptions = new FileUploadOptions(
            'image', // fileKey
            app.apps.profile.newProfilePhoto, // fileName
            'image/jpeg', // mimeType
            /* jshint -W106 */
            { account_id: app.auth.getAccountId(), account_hash: app.auth.getAccountHash() }
            /* jshint +W106 */
        );

        fileTransfer = new FileTransfer();
        fileTransfer.upload(
            app.apps.profile.newProfilePhoto,
            encodeURI(app.apps.profile.apiRoute + 'upload_avatar'),
            success,
            error,
            fileOptions
        );
    },

    logout: function () {

        function confirmHandler(response) {
            var confirmed = 1;

            if (response === confirmed) {
                app.auth.logout();
            }
        }

        window.navigator.notification.confirm('Following action will log out from Kosmos App.', confirmHandler, 'Log out', ['Log out', 'Cancel']);
    },

    renderOwnProfile: function (data) {
        data = util.getAjaxData(data);
        var injectPointerEl = this.subpage === 'activity' ? '#profile-info' : '#profile-info-likes-activity';

        if (data.status === 'ok') {
            if (!this.loadMore) {
                if (data.data.avatar) {
                    app.apps.profile.updateAvatar(data.data.avatar);
                }

                data.data.canEdit = true;
                data.data.subpage = this.subpage;
                data.data.showActivitySwitcher = true;

                ui.render('profile-info', data.data, injectPointerEl);
            }

            $('.profile-title').text(app.apps.profile.getNickname());
            app.apps.communities.getUserActivity(app.auth.getAccountId(), this.from, this.limit, this.insertPosition, this.subpage);
        } else if (data.status === 'err') {
            plugins.toast.showShortBottom(data.data);
        } else {
            plugins.toast.showShortBottom('Server error occurred');
        }
    },

    getOwnProfile: function (from, limit, insertPosition, cb, loadMore, subpage) {
        cb = cb ? cb : function () {};
        loadMore = loadMore ? loadMore : false;
        from = from ? from : 0;
        limit = limit ? limit : 20;
        insertPosition = insertPosition ? insertPosition : null;
        subpage = subpage ? subpage : 'activity';

        var ajaxSettings = {
            url:  app.apps.profile.apiRoute + 'get/' + app.auth.getAccountId(),
            insertPosition: insertPosition,
            limit: limit,
            from: from,
            loadMore: loadMore,
            subpage: subpage,
            /* jshint -W106 */
            data: {
                id: app.auth.getAccountId(),
                get_communities_info: true
            }
            /* jshint +W106 */
        };

        app.request.make(ajaxSettings, cb);
    },

    goToPublicProfile: function (profileId) {
        // Trying to open self profile so use different logic
        if (parseInt(profileId, 10) === parseInt(app.auth.getAccountId(), 10)) {
            app.tabs.open('you');
            return;
        }
        app.apps.profile.currentProfileId = profileId;
        app.refresh.updateFrom(null, 'page-app-profile-public', true);
        app.apps.profile.loadPublicProfile(null, null, null, app.apps.profile.loadPublicProfileCallback, false);
        app.spinner.render('#profile-public-info');
        app.navigate('page-app-profile-public');
    },

    loadPublicProfile: function (from, limit, insertPosition, cb, loadMore) {
        cb = cb ? cb : function () {};
        from = from ? from : 0;
        limit = limit ? limit : 20;
        loadMore = loadMore ? loadMore : false;
        insertPosition = insertPosition ? insertPosition : 'innerHTML';

        var ajaxSettings = {
            url:  app.apps.profile.apiRoute + 'get/' + app.apps.profile.currentProfileId,
            profileId: app.apps.profile.currentProfileId,
            from: from,
            limit: limit,
            insertPosition: insertPosition,
            loadMore: loadMore,
            /* jshint -W106 */
            data: { id: app.auth.getAccountId(), get_communities_info: true }
            /* jshint +W106 */
        };

        app.request.make(ajaxSettings, cb);
    },

    loadPublicProfileCallback: function (data) {
        data = util.getAjaxData(data);

        if (data.status === 'ok') {
            if (!this.loadMore) {
                ui.render('profile-info', data.data, '#profile-public-info');
            }

            /* jshint -W106 */
            document.getElementById('profile-public-title').innerText = data.data.nickname || data.data.first_name;
            /* jshint +W106 */
            app.apps.communities.getUserActivity(this.profileId, this.from, this.limit, this.insertPosition);
        } else if (data.status === 'err') {
            plugins.toast.showShortBottom(data.data);
        } else {
            plugins.toast.showShortBottom('Server error occurred');
        }
    },

    goToEditOwnProfile: function () {
        var ajaxSettings = {
            url: app.apps.profile.apiRoute + 'get/' + app.auth.getAccountId(),
            data: {}
        };

        var onSuccess = function (data) {
            data = util.getAjaxData(data);

            if (data.status === 'ok') {
                if (data.data.avatar) {
                    app.apps.profile.updateAvatar(data.data.avatar);
                }

                ui.render('profile-edit', data.data, '#profile-edit');
            }
        };

        app.navigate('page-app-profile-edit');
        app.spinner.render('#profile-edit');

        app.request.make(ajaxSettings, onSuccess);
    },

    setProfileCommunityId: function (communityId) {
        app.storage.local.set('profileCommunityId', communityId);
    },

    getProfileCommunityId: function () {
        return app.storage.local.get('profileCommunityId');
    },

    requestInitProfileData: function (cb) {
        if (app.auth.getAccountId()) {
            var ajaxSettings = { url: app.apps.profile.apiRoute + 'get/' + app.auth.getAccountId() };

            var success = function (data) {
                data = util.getAjaxData(data);

                if (data.status === 'ok') {
                    console.log('profile init finished with data: ', data);
                    //setAvatar
                    if (data.data.avatar) {
                        app.apps.profile.updateAvatar(data.data.avatar);
                    }

                    //setNickname
                    if (data.data.nickname !== '') {
                        app.apps.profile.setNickname(data.data.nickname);
                    }

                    /* jshint -W106 */
                    if (data.data.account_id === app.auth.getAccountId()) {
                        app.apps.profile.setProfileCommunityId(data.data.profile_community_id);
                    }
                    /* jshint +W106 */

                    if (cb) {
                        cb();
                    }
                }
            };

            app.request.make(ajaxSettings, success);
        } else {
            setTimeout(app.apps.profile.requestInitProfileData, 500);
        }
    },

    saveProfile: function () {
        var ajaxSettings = {
            url: app.apps.profile.apiRoute + 'update',
            data: {}
        };

        var fields = ['email', 'nickname', 'first_name', 'last_name', 'place', 'phone_number', 'about'];
        var el;

        for (var i = 0, len = fields.length; i < len; i++) {
            el = document.getElementById(fields[i]);
            if (el.getAttribute('disabled') === 'disabled') {
                continue;
            }
            ajaxSettings.data[fields[i]] = el.value;
        }

        var success = function (data) {
            data = util.getAjaxData(data);

            if (data.status === 'ok') {
                ui.render('profile-edit', data.data, '#profile-edit');
                plugins.toast.showShortBottom('Your profile settings successfully saved');

                if (typeof data.data.nickname !== 'undefined') {
                    app.apps.profile.setNickname(data.data.nickname);
                }

                app.analytics.mx.track('profile-updated');
            } else {
                plugins.toast.showShortBottom('Cannot save profile: ' + data.data);
            }
        };

        app.request.make(ajaxSettings, success);
    },

    privateMessage: function (id, username) {
        var participants = {
            participants: [app.auth.getAccountId(), id]
        };
        app.apps.messenger.startConversation(null, participants);
        app.apps.messenger.changeTitle(username);
    },

    setNickname: function (nickname) {
        this.nickname = nickname;
        return app.storage.local.set('nickname', nickname);
    },

    getNickname: function () {
        if (this.nickname === null) {
            this.nickname = app.storage.local.get('nickname');
        }
        //Fallback to name if nickname is not available
        if (this.nickname === null) {
            this.nickname = app.auth.getName();
        }
        return this.nickname;
    },

    updateAvatar: function (newAvatar) {
        app.apps.profile.avatar = newAvatar;

        if (!newAvatar) {
            console.error('Cannot update avatar since newAvatar passed is invalid.');
        }

        var elAvatarsToUpdate = $('.updateable-avatar');

        elAvatarsToUpdate.each(function (idx, el) {
            if (el.tagName === 'IMG') {
                el.src = newAvatar;
            } else {
                el.style.backgroundImage = 'url(' + newAvatar + ')';
            }
        });
    },

    //////////////////////////////////////////////////
    //              NAVIGATION LISTENERS            //
    //////////////////////////////////////////////////
    _beforeNavigateEvent: function (e) {},

    //////////////////////////////////////////////////
    //              BACK BUTTON LISTENERS           //
    //////////////////////////////////////////////////
    _backEvent: function (e) {
        if (location.hash.indexOf('#page-app-profile') !== -1) {
            if (e) {
                e.preventDefault();
            }

            app.history.back();
        }
    }
};
